<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Helper;
use App\Models\Post;
use App\Models\PostVersion;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Symfony\Component\HttpFoundation\Response;

class PostController extends Controller
{
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'feature' => 'boolean',
            'priority' => 'numeric',
            'post_title' => 'required|string',
            'description' => 'required|string',
            'tags' => 'required|array',
            'tags.*' => 'required|numeric',
            'type_face' => 'required|array',
            'type_face.*' => 'required|numeric',
            'website' => 'required|string',
            'permlink' => 'string',
            'anchor_website' => 'string',
            'version_number' => 'required|numeric',
            'version_title' => 'required|string',
            'version_date_added' => 'required|date',
            'stack' => 'array',
            'stack.*' => 'numeric',
            'version_picture' => 'required|array',
            'version_picture.*.' => 'required|image',
            'version_figma_file' => 'file',
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        $input = Helper::FormPostData($request, $request->user()->id);
        $post = Post::create($input);
        $data = [];
        $keys = $request->only('tags', 'type_face');
        foreach ($keys as $key => $values) {
            foreach ($values as $value) {
                $data[] = ['category_id' => $value, 'post_id' => $post->id];
            }
        }
        if (count($data) > 0)
            DB::table('post_category')->insert($data);

        $pv = PostVersion::create([
            'title' => $request->version_title,
            'version_number' => $request->version_number,
            'post_id' => $post->id,
            'date_added' => $request->version_date_added,
        ]);
        if ($request->has('stack')) {
            $data = [];
            foreach ($request->stack as $value) {
                $data[] = ['post_version_id' => $pv->id, 'category_id' => $value];
            }
            DB::table('version_categories')->insert($data);
        }
        if ($request->has('version_picture')) {
            foreach ($request->version_picture as $key => $value) {
                Helper::storeFile([['file' => $value, 'info' => ['type' => 'PostVersion', 'section' => 'version_picture', 'sub_section' => '', 'item_id' => $pv->id, 'index' => $key]]]);
            }
        }

        if ($request->has('version_figma_file')) {
            Helper::storeFile([['file' => $request->file('version_figma_file'), 'info' => ['type' => 'PostVersion', 'section' => 'version_figma_file', 'sub_section' => '', 'item_id' => $pv->id, 'index' => null]]]);
        }
        return Helper::return(['message' => __('general.created', ['entity' => 'Post'])], Response::HTTP_OK);
    }
    public function show($id)
    {
        $post = Post::with('categories', 'versions.category', 'versions.files')->find($id);
        if (count($post->categories) < 1) {
            return Helper::return(['message' => __('general.fetched', ['entity' => 'Post']), 'data' => ['post_info' => $post, 'related_posts' => null]], Response::HTTP_OK);
        }
        $this_post_categories = [];
        foreach ($post->categories as $category) {
            $this_post_categories[] = $category->id;
        }

        $related = Post::whereHas('categories', function ($q) use ($this_post_categories) {

            $q->where('categories.id', $this_post_categories);
        })->get();
        return Helper::return(['message' => __('general.fetched', ['entity' => 'Post']), 'data' => ['post_info' => $post, 'related_posts' => $related]], Response::HTTP_OK);
    }
    public function showAll(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'search' => 'string',
            'sort' => 'string|in:created_at,published_date,a-z',
            'category' => 'string',
            'paginate' => 'numeric',
            'sort_type' => 'in:ASC,DESC',
            'status' => 'in:Draft,Published,Submission',
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        $model = new Post;
        if ($request->has('search'))
            $model = $model->where('title', "LIKE", "%$request->search%");
        if ($request->has('category')) {
            $model = $model->whereHas('categories', function ($query) use ($request) {
                $query->where('name', $request->category);
            });
        }
        if ($request->has('status'))
            $model = $model->where('status', $request->status);

        if ($request->has('sort')) {
            $request->has('sort_type') ? $type = $request->sort_type : $type = 'ASC';
            if ($request->sort == 'a-z') {
                $model = $model->orderBy('title', $type);
            } else {
                $model = $model->orderBy($request->sort, $type);
            }
        }
        $data = $model->paginate($request->paginate);

        return Helper::return(['message' => __('general.fetched', ['entity' => 'Post']), 'data' => $data], Response::HTTP_OK);
    }
    public function takeAction(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
            'action' => 'required|in:draft,publish,delete',
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        if (!$post = Post::find($id))
            return Helper::return(['message' => __('general.invalid_id', ['entity' => 'Post']), 'data' => []], Response::HTTP_OK);
        if ($request->action == 'publish')
            $post->update(['published_date' => Carbon::now(), 'status' => 'Published']);
        if ($request->action == 'draft')
            $post->update(['published_date' => null, 'status' => 'Draft']);
        if ($request->action == 'delete')
            $post->delete();
        return Helper::return(['message' => __('general.successfuly_updated', ['entity' => 'Post']), 'data' => []], Response::HTTP_OK);
    }

    public function index(Request $request)
    {
        $model = new Post;
        $model = $model->where('status', 'Published');
        if ($request->has('tags'))
            $model = $model->orWhereHas('categories', function ($query) use ($request) {
                $query->where('name', $request->tags);
            });
        if ($request->has('type_face')) {
            $model = $model->orWhereHas('categories', function ($query) use ($request) {
                $query->where('name', $request->type_face);
            });
        }
        $data = $model->with('categories.detail', 'versions.category', 'versions.files')->get();
        $bookmarks = BookmarkController::mine();
        $data = BookmarkController::addBookmarksToData($data, $bookmarks);

        return Helper::return(['message' => __('general.fetched', ['entity' => 'Post']), 'data' =>  $data], Response::HTTP_OK);
    }
}
